@@ -1,17 +1,17 @@
 `ensembleBMA` <-
-function(ensembleData, dates = NULL, trainingRule = list(length=NA, lag=NA), 
-         control = NULL, model = NULL, exchangeable = NULL, minCRPS = NULL)
+function(ensembleData, trainingDays, dates = NULL, control = NULL, 
+         model = NULL, exchangeable = NULL, minCRPS = NULL)
 {
  if (!inherits(ensembleData,"ensembleData")) stop("not an ensembleData object")
  mc <- match.call()   
  mc$model <- NULL
 
- if (!is.null(model)) {
-   MODELS <- c("normal","gamma0")
-   m <- pmatch( model, MODELS, nomatch=0)
-   model <- if (m) MODELS[m] else "?"
- }
- else stop("unspecified model")
+## if (!is.null(model)) {
+##   MODELS <- c("normal","gamma0")
+##   m <- pmatch( model, MODELS, nomatch=0)
+##   model <- if (m) MODELS[m] else "?"
+##}
+##else stop("unspecified model")
 
   switch( model,
         "normal" = {
@@ -24,6 +24,13 @@ function(ensembleData, dates = NULL, trainingRule = list(length=NA, lag=NA),
                mc$minCRPS <- NULL
              }
          },
+        "gamma" = {
+             mc[[1]] <- as.name("ensembleBMAgamma")
+             if (!is.null(minCRPS)) {
+               if (minCRPS) warning("minCRPS not available for gamma model")
+               mc$minCRPS <- NULL
+             }
+         },
        stop("unrecognized model")
     )
 
